unit mainDemoReport;

{$mode objfpc}{$H+}

{$define ColorBands}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls, ComCtrls, fpreport;

const
  TESTIMAGE = 'testimage.png';

type

  { TForm1 }

  TForm1 = class(TForm)
    BuRenderCanvas: TButton;
    BuRenderPrerview: TButton;
    BuRenderPdf: TButton;
    lblPage: TLabel;
    PanelRender: TPanel;
    panelMain: TPanel;
    UpDown1: TUpDown;
    procedure BuRenderCanvasClick(Sender: TObject);
    procedure BuRenderPdfClick(Sender: TObject);
    procedure BuRenderPrerviewClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PanelRenderPaint(Sender: TObject);
    procedure UpDown1Changing(Sender: TObject; var AllowChange: Boolean);
  private
    FInit: Boolean;
    // Report
    FReport: TFPReport;
    FDataParent : TComponent;
    lReportData : TFPReportUserData;
    sl: TStringList;
    // Exporter
    rptExporter : TFPReportExporter;
    procedure CreateDemoReport;
    procedure GetReportDataEOF(Sender: TObject; var IsEOF: Boolean);
    procedure GetReportDataFirst(Sender: TObject);
    procedure GetReportDataNames(Sender: TObject; List: TStrings);
    procedure GetReportDataValue(Sender: TObject; const AValueName: String;
      var AValue: Variant);
    procedure InitialiseData;
    procedure CleanUp;
    procedure ButtonSet(state:Boolean);
    procedure LCLPaint(sender: TObject);
  public

  end;

var
  Form1: TForm1;

implementation

uses
  fpTTF,
  fpreportformexport,
  fpreportlclexport,
  fpreportpdfexport;

{$R *.lfm}

procedure TForm1.PanelRenderPaint(Sender: TObject);
begin
  rptExporter.Execute;
end;

procedure TForm1.UpDown1Changing(Sender: TObject; var AllowChange: Boolean);
begin
  if (UpDown1.Position > UpDown1.Max)
     or (UpDown1.Position < UpDown1.Min) then begin
    AllowChange:=false;
    exit;
  end;
  TFPReportExportCanvas(rptExporter).PageIndex:=UpDown1.Position;
  lblPage.Caption:= IntToStr(TFPReportExportCanvas(rptExporter).PageIndex);
  Invalidate;
end;

procedure TForm1.BuRenderCanvasClick(Sender: TObject);
begin
  ButtonSet(false);
  try
    CreateDemoReport;
    FReport.RunReport;

    if Assigned(rptExporter) then
      FreeAndNil(rptExporter);
    rptExporter:= TFPReportExportCanvas.Create(nil);
    rptExporter.Report:= FReport;
    rptExporter.AutoRun:=True;
    TFPReportExportCanvas(rptExporter).Canvas:= PanelRender.Canvas;
    FReport.RenderReport(rptExporter);
    UpDown1.Min:= 0;
    UpDown1.Max:= TFPReportExportCanvas(rptExporter).PageCount-1;
    UpDown1.Position:= 0;
    TFPReportExportCanvas(rptExporter).PageIndex:=UpDown1.Position;
    lblPage.Caption:= IntToStr(TFPReportExportCanvas(rptExporter).PageIndex);
    PanelRender.OnPaint:= @PanelRenderPaint;
    Invalidate;
  finally
    ButtonSet(true);
  end;
end;

procedure TForm1.BuRenderPdfClick(Sender: TObject);
begin
  ButtonSet(false);
  try
    CreateDemoReport;
    FReport.RunReport;

    if Assigned(rptExporter) then
      FreeAndNil(rptExporter);
    rptExporter:= TFPReportExportPDF.Create(nil);
    TFPReportExportPDF(rptExporter).FileName:= ApplicationName+'-report.pdf';
    rptExporter.Report:= FReport;
    rptExporter.AutoRun:=True;
    FReport.RenderReport(rptExporter);
  finally
    ButtonSet(true);
  end;
end;

procedure TForm1.BuRenderPrerviewClick(Sender: TObject);
begin
  ButtonSet(false);
  try
    CreateDemoReport;
    FReport.RunReport;

    if Assigned(rptExporter) then
      FreeAndNil(rptExporter);
    rptExporter:= TFPreportPreviewExport.Create(nil);
    rptExporter.Report:= FReport;
    rptExporter.AutoRun:=True;
    FReport.RenderReport(rptExporter);
  finally
    ButtonSet(true);
  end;
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
  if not FInit then begin
    gTTFontCache.ReadStandardFonts;
    gTTFontCache.BuildFontCache;
    FInit:= true;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FReport:= nil;
  FDataParent:= nil;
  lReportData:= nil;
  rptExporter:= nil;
  sl:= nil;
  FInit:= False;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  CleanUp;
end;


procedure TForm1.CleanUp;
begin
  if Assigned(rptExporter) then FreeAndNil(rptExporter);
  if Assigned(FReport) then FreeAndNil(FReport);
  if Assigned(FDataParent) then FreeAndNil(FDataParent);
  if Assigned(lReportData) then FreeAndNil(lReportData);
  if Assigned(sl) then FreeAndNil(sl);
end;

procedure TForm1.ButtonSet(state: Boolean);
begin
  BuRenderCanvas.Enabled:= state;
  BuRenderPrerview.Enabled:= state;
  BuRenderPdf.Enabled:= state;
  Application.ProcessMessages;
end;

procedure TForm1.LCLPaint(sender: TObject);
begin

end;


procedure TForm1.CreateDemoReport;

var
  p: TFPReportPage;
  TitleBand: TFPReportTitleBand;
  DataBand: TFPReportDataBand;
  Memo: TFPReportMemo;
  PageFooter: TFPReportPageFooterBand;
  SummaryBand: TFPReportSummaryBand;
begin
  CleanUp;

  lReportData := TFPReportUserData.Create(Self);
  lReportData.Name:='UserData';
  lReportData.OnGetValue := @GetReportDataValue;
  lReportData.OnGetEOF := @GetReportDataEOF;
  lReportData.OnFirst := @GetReportDataFirst;
  lReportData.OnGetNames := @GetReportDataNames;
  lReportData.InitFieldDefs;
  InitialiseData;

  FReport:=TFPReport.Create(Self);
  FReport.Author := 'Graeme Geldenhuys';
  FReport.Title := 'FPReport Demo 1 - Simple Listing';
  FReport.Variables.AddVariable('Var1').AsString:='Value1';
  FReport.Variables.AddVariable('Var2').AsString:='Value2';

  p := TFPReportPage.Create(FReport);
  p.Orientation := poLandscape;
  p.PageSize.PaperName := 'A4';
  { page margins }
  p.Margins.Left := 30;
  p.Margins.Top := 20;
  p.Margins.Right := 30;
  p.Margins.Bottom := 20;
  p.Data := lReportData;

  TitleBand := TFPReportTitleBand.Create(p);
  TitleBand.Layout.Height := 40;
  {$ifdef ColorBands}
  TitleBand.Frame.Shape := fsRectangle;
  TitleBand.Frame.BackgroundColor := clReportTitleSummary;
  {$endif}

  Memo := TFPReportMemo.Create(TitleBand);
  Memo.Layout.Left := 55;
  Memo.Layout.Top := 20;
  Memo.Layout.Width := 50;
  Memo.Layout.Height := 10;
  Memo.Font.Name := 'LiberationSans';
  Memo.Text := 'THE REPORT TITLE';

  With TFPReportShape.Create(TitleBand) do
    begin
    Layout.Left := 5;
    Layout.Top := 15;
    Layout.Width := 20;
    Layout.Height := 20;
    ShapeType:=stCircle;
    end;
  With TFPReportCheckbox.Create(TitleBand) do
    begin
    Layout.Left := 35;
    Layout.Top := 15;
    Layout.Width := 5;
    Layout.Height := 5;
    end;

  With TFPReportImage.Create(TitleBand) do
    begin
    Layout.Left := 5;
    Layout.Top := 2;
    Layout.Width := 35;
    Layout.Height := 35;
    Stretched:=True;
    LoadFromFile(ExtractFilePath(ParamStr(0))+TESTIMAGE);
    end;


  DataBand := TFPReportDataBand.Create(p);
  DataBand.Layout.Height := 10;
  {$ifdef ColorBands}
  DataBand.Frame.Shape := fsRectangle;
  DataBand.Frame.BackgroundColor := clDataBand;
  {$endif}

  Memo := TFPReportMemo.Create(DataBand);
  Memo.Layout.Left := 5;
  Memo.Layout.Top := 0;
  Memo.Layout.Width := 60;
  Memo.Layout.Height := 5;
  Memo.Font.Name := 'LiberationSans';
  Memo.Text := 'Hello world <[userdata.string]>.';

  PageFooter := TFPReportPageFooterBand.Create(p);
  PageFooter.Layout.Height := 40;
  {$ifdef ColorBands}
  PageFooter.Frame.Shape := fsRectangle;
  PageFooter.Frame.BackgroundColor := clPageHeaderFooter;
  {$endif}

  Memo := TFPReportMemo.Create(PageFooter);
  Memo.Layout.Left := 135;
  Memo.Layout.Top := 13;
  Memo.Layout.Width := 15;
  Memo.Layout.Height := 5;
  Memo.Options:=[moDisableWordWrap];
  Memo.Font.Name := 'LiberationSans';
  Memo.Text := 'Page [PageNo] '+ExtractFilePath(ParamStr(0))+TESTIMAGE;

  With TFPReportImage.Create(PageFooter) do
    begin
    Layout.Left := 5;
    Layout.Top := 2;
    Layout.Width := 35;
    Layout.Height := 35;
    Stretched:=True;
    LoadFromFile(ExtractFilePath(ParamStr(0))+TESTIMAGE);
    end;

  SummaryBand := TFPReportSummaryBand.Create(p);
  SummaryBand.Layout.Height := 40;
  {$ifdef ColorBands}
  SummaryBand.Frame.Shape := fsRectangle;
  SummaryBand.Frame.BackgroundColor := clReportTitleSummary;
  {$endif}

  Memo := TFPReportMemo.Create(SummaryBand);
  Memo.Layout.Left := 19;
  Memo.Layout.Top := 10;
  Memo.Layout.Width := 70;
  Memo.Layout.Height := 25;
  Memo.Font.Name := 'LiberationSans';
  Memo.StretchMode := smActualHeight;
  Memo.Text := 'This is some long text that should be wrapping inside the memo. It has a 10mm left margin, and a 7mm right margin. 0mm margin top and bottom.';
  Memo.TextAlignment.LeftMargin := 10;
  Memo.TextAlignment.RightMargin := 7;
  Memo.Frame.Shape := fsRectangle;
  Memo.Frame.BackgroundColor := clLtGray;
end;

procedure TForm1.GetReportDataFirst(Sender: TObject);
begin

end;

procedure TForm1.GetReportDataValue(Sender: TObject;
  const AValueName: String; var AValue: Variant);
begin
  if (AValueName = 'element') or (AValueName = 'string') then
  begin
    AValue := sl[lReportData.RecNo-1];
  end
  else
    AValue:=AValueName+IntToStr(lReportData.RecNo);
end;

procedure TForm1.GetReportDataEOF(Sender: TObject;
  var IsEOF: Boolean);
begin
  if lReportData.RecNo > sl.Count then
    IsEOF := True
  else
    IsEOF := False;
end;

procedure TForm1.GetReportDataNames(Sender: TObject;
  List: TStrings);
begin
  List.Add('element');
  List.Add('string');
  List.Add('FirstName');
  List.Add('LastName');
  List.Add('DateOfBirth');
  List.Add('Gender');
  List.Add('Email');
end;

procedure TForm1.InitialiseData;
var
  i: integer;
begin
  sl := TStringList.Create;
  for i := 1 to 5 do
    sl.Add(Format('Item %d', [i]));
end;





end.

